/*
 *	linux/arch/arm/mach-nspire/mmio.h
 *
 *	Copyright (C) 2013 Daniel Tang <tangrs@tangrs.id.au>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2, as
 * published by the Free Software Foundation.
 *
 */

#define NSPIRE_MISC_PHYS_BASE		0x900A0000
#define NSPIRE_MISC_HWRESET		0x08

#define NSPIRE_PWR_PHYS_BASE		0x900B0000
#define NSPIRE_PWR_VIRT_BASE		0xFEEB0000
#define NSPIRE_PWR_BUS_DISABLE1		0x18
#define NSPIRE_PWR_BUS_DISABLE2		0x20

#define NSPIRE_LCD_PHYS_BASE		0xC0000000
